import os

__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))


def fixture_content(filename):
    path = os.path.join(__location__, "fixtures", filename)
    if not os.path.exists(path):
        return ""
    with open(path) as f:
        return f.read()



import goifer
import os
from pprint import pprint
import requests

client = goifer.Client("canton_zurich")
member = client.search(
    index="Mitglieder",
    query='Wahlkreis adj "IX Horgen" and dauer_start > "2019-11-19 00:00:00"',
)

member = member[0]
pprint(member)

# download image

foto_id = member["foto_id"]
version = member["foto_version"]["nr"]
view = member["foto_version"]["rendition"][0]["ansicht"]
ext = member["foto_version"]["rendition"][0]["extension"]

download_path = os.path.join(".", f"foto.{ext}")
download_url = client.file("Mitglieder", foto_id, version, view)

r = requests.get(download_url)
print(f"Download file {download_url}...")
with open(download_path, "wb") as f:
    for chunk in r.iter_content(1024):
        f.write(chunk)

try:
    from PIL import Image

    img = Image.open(download_path)
    img.show()
except ImportError:
    print(f"Pillow is not installed, to view the photo, open '{download_path}'")



import goifer
from pprint import pprint

records = goifer.search(
    "canton_zurich",
    index="Wahlkreise",
    query="inaktiv = false sortBy name/sort.ascending",
)
print("")

all_records = [r for r in records]
pprint(all_records)



import goifer
import requests
from pprint import pprint

# sometime is't necessary to tweak the requests Session (e.g. to provide authentication, set a custom header or disable SSL verification). For this purpose a customized session can be passed to the Client

session = requests.Session()
session.verify = False  # disable SSL verification

client = goifer.Client("canton_zurich", maximum_records=1000, session=session)
records = client.search(index="Ablaufschritte", query="seq>2075771")

print(records)
pprint(records[0])



import pytest
from conftest import fixture_content
from goifer.client import Client
import responses


class TestClient:
    @pytest.fixture
    def wahlkreise(self):
        return fixture_content("wahlkreise.xml")

    @responses.activate
    def test_indexes(self):
        client = Client("canton_zurich")
        indexes = client.indexes()
        assert isinstance(indexes, list), "indexes is not a list"
        assert len(indexes) == 11

    @responses.activate
    def test_search_wahlkreis(self, wahlkreise):
        responses.add(
            responses.GET,
            "https://parlzhcdws.cmicloud.ch/parlzh2/cdws/Index/WAHLKREISE/searchdetails",
            content_type="text/xml",
            body=wahlkreise,
            status=200,
        )
        client = Client("canton_zurich")
        results = client.search("Wahlkreise", "seq > 0")
        wahlkreis = results[0]

        assert wahlkreis["name"] == "I      Zürich 1+2"



# -*- coding: utf-8 -*-

import requests
import yaml
import os
from . import errors
from . import response


__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))


class Client(object):
    def __init__(self, instance, maximum_records=500, session=None, config=None):
        self.maximum_records = maximum_records
        if session:
            self.session = session
        else:
            self.session = requests.Session()

        if isinstance(config, dict):
            full_config = config
        elif isinstance(config, str):
            full_config = self._load_yaml_file(config)
        else:
            # if no config is provided, load the default config
            path = os.path.join(__location__, "config.yml")
            full_config = self._load_yaml_file(path)
        try:
            self.config = full_config[instance]
        except KeyError as e:
            raise errors.ConfigError(f"Instance '{instance}' not found in config: {e}")

    def _load_yaml_file(self, path):
        with open(path, "r") as f:
            return yaml.safe_load(f)

    def indexes(self):
        return list(self.config["indexes"].keys())

    def search(self, index, query="seq > 0", start_record=1):
        index_url = self._get_index_url(index)
        url = f"{index_url}/searchdetails"
        params = {
            "q": query,
            "l": "de-CH",
            "m": self.maximum_records,
        }

        data_loader = DataLoader(url, params, self.session)
        return response.SearchResponse(data_loader, index, self.config)

    def file(self, index, file_id, version, view):
        index_config = self.config["indexes"][index]

        if "section" in index_config:
            path = f"/{index_config['section']}{self.config['files_api']['path']}"
        else:
            path = self.config["files_api"]["path"]
        url = f"{self.config['api_base']}{path}/{file_id}/{version}/{view}"
        return url

    def schema(self, index):
        index_url = self._get_index_url(index)
        url = f"{index_url}/schema"
        params = {}
        data_loader = DataLoader(url, params, self.session)
        return response.SchemaResponse(data_loader, index, self.config)[0]

    def _get_index_url(self, index):
        try:
            index = self.config["indexes"][index]

            base = self.config["api_base"]
            section = index.get("section", "")
            path = index["path"]
            if section:
                url = f"{base}/{section}{path}"
            else:
                url = f"{base}{path}"
            return url
        except KeyError as e:
            raise errors.ConfigError(f"Config error: {e}")


class DataLoader(object):
    def __init__(self, url, params, session):
        self.url = url
        self.params = params
        self.session = session
        self.response = None

    def load(self, **kwargs):
        self.params.update(kwargs)
        xml = self._get_content(self.url, self.params)
        return xml

    def _get_content(self, url, params):
        try:
            res = self.session.get(url, params=params)
            res.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise errors.GoiferError("HTTP error: %s" % e)
        except requests.exceptions.RequestException as e:
            raise errors.GoiferError("Request error: %s" % e)
        return res.content
